/**
 * @file Ce fichier génère le schéma GraphQL. C'est ici que les requêtes GraphQl sont résolues.
 * @author akka vodol
*/
import { makeExecutableSchema } from 'graphql-tools';
import { request } from 'https';
import { assertBinaryExpression } from 'babel-types';
import knex from '../../db/knex_router';
import typeDefs from './typeDefs';

import * as db_utils from './db_utils';

/** 
 * @description Résolveurs des différentes requêtes GraphQL
*/
const resolvers = {
    Query: {

        asAdmin: (obj, args, context) => {
            return db_utils.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have admin rights over this group";
            });
        },

        allGroups: (obj, args, context) => {
            return db_utils.getAllVisibleGroups(context.user);
        },
        group: (obj, args, context) => {
            return db_utils.getGroupIfVisible(context.user, args.uid);
        },
        user: (obj, args, context) => {
            const refactorer = (data) => {
                return {
                    uid: args.uid,
                    lastName: data.sn,
                    givenName: data.givenName,
                    birthdate: data.brBirthdate,
                    groups: data.brMemberOf,
                    mail: data.mail,
                    phone: data.telephoneNumber,
                    room: data.brRoom
                };
            };

            const result = db_utils.renseignerSurUtilisateur(context.user, args.uid).then(res => {
                const output = res.map(entry => refactorer(entry));
                return output;
            });

            return result;
        },

        allPosts: (obj, args, context) => {
            return knex.select().from('posts');
        },

        searchTOL: (obj, args, context) => {
            console.log(args);
            return db_utils.repliquerTOLdesIds(context.user, {
                givenName: args.givenName,
                lastName: args.lastName,
                nickname: args.nickname,
                nationality: args.nationality,
                school: args.school,
                promotion: args.promotion,
                groups: args.groups,
                studies: args.studies,
                sport: args.sport,
                phone: args.phone,
                mail: args.mail,
                adress: args.adress,
                ip: args.ip
            });
        }
    },

    AdminQuery: {
        isAdmin: (obj, args, context) => {
            return true;
        },

        allRequests: (obj, args, context) => {
            console.log(obj);
            return obj;
        }
    },

    Requests: {
        userJoinGroup : (obj, args, context) => {
            return db_utils.getUserJoinGroupRequests(context.user, args.groupUID);
        },
    },

    UserJoinGroup: {
        requestUID: (obj, args, context) => {
            return obj.id;
        },

        user : (obj, args, context) => {
            return db_utils.getUser(context.user, obj.useruid);
            /*return db_utils.getUser(context.user, "quentin.gendre");
            if(obj.useruid === "anatole.romon"){
                return db_utils.getUser(context.user, "anatole.romon").then(res => {
                    return db_utils.getUser(context.user, "quentin.gendre");
                });
            }else{
                return new Promise( (resolve, reject) => {
                    resolve({givenName : "patrick"});
                });
            }*/
        }
    },

    /*User : {

        givenName : (obj, args, context) => {
            console.log("I got this")
            return null;
        },

        lastName : (obj, args, context) => {
            return null;
        },

        uid : (obj, args, context) => {
            return null;
        },

        birthdate : (obj, args, context) => {
            return null;
        },

        mail : (obj, args, context) => {
            return null;
        },

        phone : (obj, args, context) => {
            return null;
        },

        groups : (obj, args, context) => {
            return null;
        },

        likes : (obj, args, context) => {
            return null;
        },

        address : (obj, args, context) => {
            return null;
        }

    },*/

    Mutation: {
        asAdmin: (obj, args, context) => {
            return db_utils.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have admin rights over this group";
            });
        },

        asSpeaker: (obj, args, context) => {
            return db_utils.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have publishing rights in this group";
            });
        },

        asMember: (obj, args, context) => {
            return db_utils.isMember(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        },
        asViewer: (obj, args, context) => {
            return db_utils.getGroupIfVisible(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        }
    },

    AdminMutation: {
        isAdmin: (obj, args, context) => {
            return true;
        
        },
        createSubgroup: (obj, args, context) => {
            args.parentuid = obj.groupUID;
            return db_utils.createSubgroup(context.user, args);
        },

        editGroup: (obj, args, context) => {
            return null;
        }
    },

    ViewerMutation: {
        requestJoin: (obj, args, context) => {
            console.log(obj);
            return null;
        }
    },

    Post: {
        authors: (obj, args, context) => {
            console.log("Authors of post are:",obj.authors);
            return knex.select().from('groups').whereIn('uid',obj.authors);
        }
    }
};

const schema = makeExecutableSchema({
    typeDefs,
    resolvers
});

export default schema;