const RootTypes = `
    # Requêtes
    type Query {
        allGroups: [Group]
        group(uid: ID) : Group
        user(uid: ID) : [User]
        allMessages: [Message]
        allEvents: [Event]
        allPosts: [Post]
        allAnnouncements: [Announcement]

        asAdmin(groupUID: ID): AdminQuery

        
        searchTOL(
            givenName: String,
            lastName: String,
            nickname: String,
            nationality: String,
            school: String,
            promotion: String,
            groups: String,
            studies: String,
            sport: String,
            phone: String,
            mail: String,
            adress: String,
            ip: String
        ): [String]
    }

    type Mutation {
        asAdmin(groupUID: String): AdminMutation
        asSpeaker(groupUID: String): SpeakerMutation
        asMember(groupUID: String): MemberMutation
        asViewer(groupUID: String): ViewerMutation
    }

   
`;

const subMutations = `
    type AdminMutation {

        isAdmin: Boolean

        createSubgroup(
            uid: ID = null,
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

        addUser(userid : String): User
        removeUser(userid : String): User
        addAdmin(userid : String): User
        removeAdmin(userid : String): User

        editGroup(
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

    }

    type SpeakerMutation{
        postEvent(name: String, date: String): Event
    }

    type MemberMutation {
        leave: Group
    }

    type ViewerMutation {
        requestJoin: Group
    }

`;

const subQueries = `
    # Requête à la base de donnée nécessitant d'être administrateur.
    type AdminQuery{
        isAdmin: Boolean
        allRequests : Requests
    }
`;

const User = `
    # Utilisateurs
    type User {
        # Prénom de l'utilisateur
        givenName: String!
        # Nom de famille
        lastName: String!
        uid: ID!
        birthdate: String!
        mail: String
        phone: String
        # Groupes dont l'utilisateur est membre.
        groups: [Group]
        # Groupes que l'utilisateur aime.
        likes: [Group]
        address: String
    }
`;

const Group = `
    # Groupes associatifs
    type Group {
        # Identifiant unique du groupe
        uid: ID
        # Nom du groupe
        name: String
        # Site du groupe
        website: String
        # Jour et heure de création du groupe
        createdAt: String!
        updatedAt: String!
        description: String
        # École d'origine du groupe
        school: String
        # Groupe parent
        parent: Group
    }
`;

const Message = `
    # Tout type de message créé par un ou plusieurs groupes.
    interface Message {
        id: ID!
        # Titre du message
        title: String!
        createdAt: String!
        updatedAt: String!
        # Auteur(s) du message
        authors: [Group]
    }
`;

const Post = `
    # Publication postée par un ou plusieurs groupes.
    type Post implements Message {
        id: ID!
        title: String!
        createdAt: String!
        updatedAt: String!
        authors: [Group]
        content: String!
    }
`;

const Announcement = `
    # Annonce publique effectuée par un ou plusieurs groupes.
    type Announcement implements Message {
        id: ID!
        title: String!
        createdAt: String!
        updatedAt: String!
        authors: [Group]
        description: String!
        views: Int
    }
`;

const Event = `
    # Événements organisés par un ou plusieurs groupes.
    type Event implements Message {
        id: ID!
        # Intitulé de l'événement
        title: String!
        # Lieu de l'événement
        location: String
        createdAt: String!
        updatedAt: String!
        startTime: String!
        endTime: String!
        # Organisateurs
        authors: [Group]
        # Personnes qui participent à l'événement.
        participants: [User]
        description: String
    }
`;

const Requests = `
    # Demandes effectuées au groupe.
    type Requests {
        userJoinGroup: [UserJoinGroup]
        groupJoinEvent: [GroupJoinEvent]
        yourGroupHostEvent: [YourGroupHostEvent]
    }

    # Demande d'un utilisateur désirant rejoindre le groupe.
    type UserJoinGroup {
        # Émetteur de la demande
        requestUID: ID
        user: User
        message: String
    }


    # Demande d'un groupe voulant rejoindre un événement
    type GroupJoinEvent {
        # Événement concerné
        event: Event
        # Groupe voulant rejoindre l'événement
        groupWantingToJoin: Group
    }

    # Demande au récipiendaire de rejoindre l'organisation d'un événement.
    type YourGroupHostEvent {
        event: Event
        sender: Group
    }

`;

const typeDefs = [RootTypes, Group, User, Message, Post, Announcement, Event, subMutations, subQueries, Requests];
export default typeDefs;
